var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
import throttle from 'lodash.throttle';
import { Action, Getter } from './decorators';
import { firebaseModule } from './module';
import { FIREBASE_INITS, FIREBASE_MUTATIONS } from './utils/dictionaries';
import { getReference } from './utils/get-reference';
export { Action, Getter, firebaseModule };
export default (function (options) {
    if (options === void 0) { options = {}; }
    return function (store) {
        FIREBASE_INITS.forEach(function (settings) {
            var _a = settings.mutations, CHILD_ADD = _a.CHILD_ADD, CHILD_CHANGE = _a.CHILD_CHANGE, CHILD_REMOVE = _a.CHILD_REMOVE, RESET = _a.RESET;
            var add = function (snapshot) {
                var key = snapshot && snapshot.key;
                var value = snapshot && snapshot.val();
                if (key && value)
                    store.commit(CHILD_ADD, { key: key, value: value });
            };
            var change = function (snapshot) {
                var key = snapshot && snapshot.key;
                var value = snapshot && snapshot.val();
                if (key && value)
                    store.commit(CHILD_CHANGE, { key: key, value: value });
            };
            var remove = function (snapshot) {
                if (snapshot)
                    store.commit(CHILD_REMOVE, snapshot.key);
            };
            var unregisterReference = function (ref) {
                ref.off('child_added', add);
                ref.off('child_changed', change);
                ref.off('child_removed', remove);
                // TODO: Properly reset. Doing a `store.commit(RESET)` will result in
                // always showing the default state...
            };
            var registerReference = function (ref) {
                // Initialize database listener
                ref.on('child_added', add);
                ref.on('child_changed', change);
                ref.on('child_removed', remove);
                // Set default data if required
                // ref.once('value', snapshot => {
                //   const value = snapshot && snapshot.val();
                //   if (snapshot && !value) ref.set(settings.defaultState);
                //   settings.initialized = true;
                // });
            };
            // Allow updates on ref change
            settings.reSync = function (oldRef, newRef) {
                // console.log('resync', oldRef, newRef);
                var oldReference = oldRef && getReference(store, settings, oldRef);
                var newReference = getReference(store, settings, newRef);
                if (oldReference)
                    unregisterReference(oldReference);
                if (newReference)
                    registerReference(newReference);
            };
            var reference = getReference(store, settings);
            if (reference)
                registerReference(reference);
            else
                store.commit(RESET);
            settings.initialized = true;
        });
        // Throttled version of the update function, this ensures that updates are
        // performed at a maximum rate of once every second.
        var updateDB = throttle(function (s, ref, m) {
            var type = m.type, payload = m.payload;
            var changeMap = s.changeMap, removeMap = s.removeMap, getters = s.getters;
            var state = store.getters[getters.STATE];
            // Only update one child
            var getId = changeMap.get(type);
            if (getId) {
                var key = getId(payload);
                return ref.child(key).set(state[key]);
            }
            // Only remove one child
            getId = removeMap.get(type);
            if (getId) {
                var key = getId(payload);
                return ref.child(key).remove();
            }
            // Synchronize the whole state
            return ref.set(state);
        }, 
        // TODO: Better throttling, this way can actually cancel sync updates. The
        // update should be throttled per branch/module and in case updated are
        // performed per child, every possible CHILD_CHANGED and CHILD_REMOVED
        // should be throttled separately.
        options.throttle == undefined ? 200 : options.throttle);
        var SETTINGS = __spread(FIREBASE_INITS.values());
        var REFERENCES = SETTINGS.map(function (s) { return s.getters.REF; });
        store.subscribe(function (mutation) {
            // All references are accessed on every mutation to ensure the value is
            // updated, invalidating the Vuex cache. This is required because references
            // need to be up to date at all times.
            REFERENCES.map(function (ref) { return store.getters[ref]; });
            var settings = FIREBASE_MUTATIONS.get(mutation.type);
            var reference = getReference(store, settings);
            if (settings && reference)
                updateDB(settings, reference, mutation);
        });
    };
});
//# sourceMappingURL=index.js.map